function [varargout] = DFP_equations(varargin) %#eml

%DFP_equations
%   This file is supplied as an addition to the draft paper:
%   "Foot Placement control: Step Location and Step Time as a
%   Function of the Desired Walking Gait"
%
%   For further information, contact:
%   Tomas de Boer, tomas.deboer@tudelft.nl, or    
%   Danil Karssen,   j.g.d.karssen@tudelft.nl
%
%   Copyright 2011, Delft BioRobotics Laboratory
%   Delft University of Technology
%   $Revision: 1.0 $  $Date: August 2011 $

[x0,dx0,x1,dx1,t,var_switch] = process_inputs(varargin);

if isempty(x1) && isempty(dx1) %case 1: x1 & dx1 unknown
    x1 = dx0*sinh(t) + x0*cosh(t);
    dx1 = dx0*cosh(t) + x0*sinh(t);
elseif isempty(x1) && isempty(x0) %case 2: x1 & x0 unknown
    x0 = (dx1 - dx0*cosh(t))/sinh(t);
    x1 = -(dx0 - dx1*cosh(t))/sinh(t);
elseif isempty(dx1) && isempty(dx0) %case 3: dx1 & dx0 unknown
    dx0 = (x1 - x0*cosh(t))/sinh(t);
    dx1 = -(x0 - x1*cosh(t))/sinh(t);
elseif isempty(x1) && isempty(dx0) %case 4: x1 & dx0 unknown
    x1 = dx1 - (2*dx1 - 2*x0*exp(t))/(exp(2*t) + 1);
    dx0 = (2*x0 + 2*dx1*exp(t))/(exp(2*t) + 1) - x0;
elseif isempty(t) && isempty(x1) %case 5: x1 & t unknown
    if (dx0 + x0) ~= 0  && (dx0 - x0) ~=0
        t = [log((dx1+(dx1^2+x0^2-dx0^2)^(1/2))/(x0+dx0));
             log((dx1-(dx1^2+x0^2-dx0^2)^(1/2))/(x0+dx0))];
        x1 = [(-dx0^2 + dx1^2 + x0^2)^(1/2);
              dx1 - (dx0^2 - x0^2)/(dx1-(-dx0^2 + dx1^2 + x0^2)^(1/2))];
        x1 = x1(imag(t)==0);
        t = t(imag(t)==0);
    elseif dx0 ~= 0
        t = log(dx1/dx0)*sign(dx0/x0);
        x1 = dx1*sign(dx0/x0);
    else
        t = [-inf;inf];
        x1 = [-dx1;dx1];
    end
elseif isempty(t) && isempty(dx1) %case 6: dx1 & t unknown
    if (dx0 + x0) ~= 0  && (dx0 - x0) ~=0 && (abs(dx0)+abs(x0))>1e-15
        t = [log((x1+(x1^2-x0^2+dx0^2)^(1/2))/(x0+dx0));
             log((x1-(x1^2-x0^2+dx0^2)^(1/2))/(x0+dx0))];
        dx1 = [(dx0^2 - x0^2 + x1^2)^(1/2);
               x1 + (dx0^2 - x0^2)/(x1 - (dx0^2 - x0^2 + x1^2)^(1/2))];
        dx1 = dx1(imag(t)==0);
        t = t(imag(t)==0);
    elseif abs(x0) > 1e-15
        t = log(x1/x0)*sign(dx0/x0);
        dx1 = x1*sign(dx0/x0);
    else
        t = [-inf,inf];
        dx1 = [-x1,x1];
    end
end

varargout = process_outputs(x0,dx0,x1,dx1,t,var_switch,varargin);

end

%% FUNCTION process_inputs
function [x0,dx0,x1,dx1,t,var_switch,nr_outputs] = process_inputs(vars)
x0 = []; dx0 = []; x1 = []; dx1 = []; t = [];

N = length(vars);
if N==7
    nr_outputs = 1;
elseif N==8
    nr_outputs = 2;
else
    error('wrong number of inputs');
end

for i=1:3
   % name = vars{i*2-1+nr_outputs};
    val = vars{i*2+nr_outputs};
    if length(val)~=1
        error('wrong input value');
    end
    switch vars{i*2-1+nr_outputs}
        case 'x0'
            x0 = val;
        case 'dx0'
            dx0 = val;
        case 'x1'
            x1 = val;
        case 'dx1'
            dx1 = val;
        case 't'
            t = val;
        otherwise
            error('wrong input name');
    end
end

var_switch = false;
if (isempty(x0)&&isempty(dx0)) || ...
        (isempty(x0)&&isempty(dx1)) || ...
        (isempty(x0)&&isempty(t)) || ...
        (isempty(dx0)&&isempty(t))
    [x0,dx0,x1,dx1,t] = switch_var(x0,dx0,x1,dx1,t);
    var_switch = true;
end
end

%% FUNCTION process_outputs
function [output] = process_outputs(x0,dx0,x1,dx1,t,var_switch,vars)
if var_switch
    [x0,dx0,x1,dx1,t] = switch_var(x0,dx0,x1,dx1,t);
end

N = length(vars);
if N==7
    nr_outputs = 1;
elseif N==8
    nr_outputs = 2;
else
    error('wrong number of inputs');
end

output = cell(nr_outputs,1);
for i=1:nr_outputs
    switch vars{i}
        case 'x0'
            output{i} = x0;
        case 'dx0'
            output{i} = dx0;
        case 'x1'
            output{i} = x1;
        case 'dx1'
            output{i} = dx1;
        case 't'
            output{i} = t;
        otherwise
            error('wrong output name');
    end
end

end

%% FUNCTION switch_var
function [x0,dx0,x1,dx1,t] = switch_var(x0,dx0,x1,dx1,t)
x0_temp = x0;
dx0_temp = dx0;
x0 = x1;
dx0 = dx1;
x1 = x0_temp;
dx1 = dx0_temp;
t = -t;
end