function [varargout]= gridfun(fun,varargin)
% GRIDFUN Apply a function to each combination input elements.
%   A = GRIDFUN(FUN, B, C, ...) applies the function specified by FUN to
%   each combination of elements of arrays B,C,... and returns the results
%   in array A. If the output of FUN is uniform A is matrix with
%   [I,J,...,N,M,...] = size(A) in which I=numel(B),J=numel(C),... and
%   [N,M,...] is the size of the output. If the output is not uniform A is
%   a cell array with [I,J,...]=size(A). The inputs B, C, ... can be
%   vectors or cell arrays.
%
%   Vectors or strings that have to be passed to the function FUN have to
%   be stored in a cell. For example, GRIDFUN(@sort,{b1,b2},1,{'descend'})
%   will sort vectors b1 and b2 in descending order.
%
%   A = GRIDFUN(FUN, B, C, ..., 'CellOutput') always outputs a cell array.
%
%   A = GRIDFUN(FUN, B, C, ..., 'ProgressIndication') shows during the
%   calculation a progress bar with an estimation of the time remaining.
%
%   A = GRIDFUN(FUN, B, C, ..., 'Squeeze') squeezes the output matrices.
%
%   A = GRIDFUN(FUN, B, C, ..., 'Transpose') transposes the output matrices.
%
%   A = GRIDFUN(FUN, B, C, ..., 'StoreResults') stores the results one a houre 
%
%   A = GRIDFUN('Resume',filename) Resumes the gridfun using the data in
%   the mat-file named filename. This can be any data file that is generated 
%   using the 'StoreResults' option.
%
%   Examples
%       function y = foo(x1,x2)
%       y = x1*x1'+x2;
%
%       gridfun(@foo,[2,3],[2,4])
%       ans =
%           6     8
%          11    13
%       gridfun(@foo,{[2,3]},[2,4])
%       ans =
%           15    17
%       gridfun(@foo,{[2,3],[4,5]},[2,4])
%       ans =
%           15    17
%           43    45
%       gridfun(@foo,{[2;3]},[2,4])
%       ans(:,:,1,1) =
%            6     8
%       ans(:,:,2,1) =
%            8    10
%       ans(:,:,1,2) =
%            8    10
%       ans(:,:,2,2) =
%           11    13
%       gridfun(@foo,{[2;3],[4,5]},[2,4])
%       ans = 
%           [2x2 double]    [2x2 double]
%           [        43]    [        45]
    
%   Copyright 2008 TUDelft, Danil Karssen & Tomas de Boer
%   Last changes Februari 16, 2010, Danil Karssen

if strcmp(fun,'Resume')
    i=0;
    load(varargin{1})
    start_i = i-1;
else
    % options
    [storeresults,varargin]       = OptionSet(varargin,'StoreResults',false);
    [progressindication,varargin] = OptionSet(varargin,'ProgressIndication',false);
    [celloutput,varargin]         = OptionSet(varargin,'CellOutput',false);
    [squeezeoutput,varargin]      = OptionSet(varargin,'Squeeze',false);
    [transposeoutput,varargin]    = OptionSet(varargin,'Transpose',false);
    
    % input and output sizes
    Nin = length(varargin); % number of inputs
    Nout = max(1,nargout); % number of outputs
    for n=1:Nin, siz(n) = numel(varargin{n}); end % size inputs
    I = prod(siz); % total number of grid points
    
    % initialize variables
    inputs = cell(Nin,1); % prealloct input cell array
    varargout = cell(Nout,1); % prealloct output cell array
    for n=1:Nout, varargout{n}=cell(I,1); end % prealloct output cell vector
    order=randperm(I);
    start_i=1;   
    start_time=0;
    last_progress=0;
    if storeresults
        save_string=['data_gridfun_',dec2hex(str2double(datestr(now,'yymmddHHMMSS'))),'_'];
    end
    
    % string for eval function
    func_str = '[';
    for n=1:Nout, func_str = [func_str, 'varargout{',int2str(n),'}{index} ']; end
    func_str = [func_str, '] = fun(inputs{:});'];
end
if progressindication
    bar_handle = waitbar(0,{' ','Please wait...',' '}); % create progress bar
end
if progressindication || storeresults, tic; end % start timer

%%% main loop
for i=start_i:I
    index = order(i);
    gridpoint = index2gridpoint(index,siz); % select grid point
    % select input
    for n=1:Nin
        if iscell(varargin{n}) % select cell if input is cell array 
            inputs{n} = varargin{n}{gridpoint(n)}; 
        else % select element if input is  
            inputs{n} = varargin{n}(gridpoint(n));
        end
    end 
    eval(func_str) % eval function

    % progress indication bar, update maximal once every second
    if progressindication && toc+start_time-last_progress>1
        last_progress = toc+start_time;
        progress(i,I,last_progress,bar_handle)
    end 
    % store data, maximal once a houre
    if storeresults && toc>3600
        start_time = toc+start_time;
        tic % restart timer
        save([save_string,datestr(now,'yyyy-mm-dd--HH-MM-SS')]);
    end
end

%%% reshape outputs
for n=1:Nout
    % if possible convert cell array into single matrix
    if ~celloutput
        % check if all outputs have the same size
        siz_output = cellfun(@size,varargout{n},'UniformOutput',0);
        if ~isempty(siz_output) && isequal(siz_output{:},siz_output{1})
            % create matrix
            dim = ndims(varargout{n}{1});
            varargout{n} = cat(dim+1,varargout{n}{:}); % create matrix A(N,M,...,I*J*...)
            varargout{n} = shiftdim(varargout{n},dim); % shift dim to A(I*J*...,N,M,...)
            varargout{n} = reshape(varargout{n},[siz,siz_output{1}]); % reshape to A(I,J,...,N,M,...)
        end
    end
    % reshape output cell vector into cell matrix
    if iscell(varargout{n}), varargout{n} = reshape(varargout{n},[siz,1]); end
    if squeezeoutput, varargout{n} = squeeze(varargout{n}); end 
    if transposeoutput, varargout{n} = varargout{n}.'; end 
end
if progressindication, close(bar_handle), end % close progress bar
if storeresults, save([save_string,datestr(now,'yyyy-mm-dd--HH-MM-SS'),'_final']); end % store final data


%%% function that checks if any options are set in the inputs
function [option,inputs] = OptionSet(inputs,option_name,default)
option = default;
if any(strcmp(inputs,option_name))
    option = true;
    inputs = inputs(~strcmp(inputs,option_name));
end

%%% function to go from index to gridpoint
function [gridpoint] = index2gridpoint(index,dims)
gridpoint = zeros(length(dims),1);
k = [1 cumprod(dims(1:end-1))];
for i = length(dims):-1:1,
  vi = rem(index-1, k(i)) + 1;
  gridpoint(i) = (index - vi)/k(i) + 1;
  index = vi;     
end

%%% fuction for progress indication
function progress(index,I,time,bar_handle)
timetogo = time/index*(I-index)/(3600*24);
if timetogo<1
    etr_text = ['Estimated time remaining: ',datestr(timetogo,13)];
else
    etr_text = ['Estimated time remaining: ',int2str(fix(timetogo)), ' day(s) and ',datestr(rem(timetogo,1),13)];
end
totaltime = time/index*I/(3600*24);
if totaltime<1
    ett_text = ['Estimated total time: ',datestr(totaltime,13)];
else
    ett_text = ['Estimated total time: ',int2str(fix(totaltime)), ' day(s) and ',datestr(rem(totaltime,1),13)];
end
finish_time = now+timetogo;
if floor(finish_time) == floor(now)
    eft_text = ['Estimated finish time: today at ', datestr(finish_time,'HH:MM:SS') ];
elseif floor(finish_time)-floor(now)<7
    eft_text = ['Estimated finish time: ', datestr(finish_time,'dddd at HH:MM:SS')];
else
    eft_text = ['Estimated finish time: ', datestr(finish_time,'ddd dd-mmm-yyyy at HH:MM:SS')];
end
waitbar(index/I, bar_handle, {etr_text,ett_text,eft_text});

%%% fuction for storing the results
function store(save_string)

